@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  bln.rb - a easyballoon compatible Saori module for ninix
-#  Copyright (C) 2002-2018 by Shyouzou Sugitani <shy@users.osdn.me>
+#  Copyright (C) 2002-2019 by Shyouzou Sugitani <shy@users.osdn.me>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -210,7 +210,7 @@ module Bln
       else
         @position = 'sakura'
       end
-      left, top, scrn_w, scrn_h = @window.workarea
+      left, top, scrn_w, scrn_h = @__sakura.get_workarea
       # -1: left, 1: right
       if @position == 'sakura'
         s0_x, s0_y, s0_w, s0_h = get_sakura_status('SurfaceSakura')
@@ -258,6 +258,7 @@ module Bln
         destroy()
         return
       end
+      @reshape = true
       @balloon_surface = balloon_surface
       w = balloon_surface.width
       h = balloon_surface.height
@@ -500,7 +501,7 @@ module Bln
     end
 
     def get_coordinate(w, h)
-      left, top, scrn_w, scrn_h = @window.workarea
+      left, top, scrn_w, scrn_h = @__sakura.get_workarea
       s0_x, s0_y, s0_w, s0_h = get_sakura_status('SurfaceSakura')
       s1_x, s1_y, s1_w, s1_h = get_sakura_status('SurfaceKero')
       b0_x, b0_y, b0_w, b0_h = get_sakura_status('BalloonSakura')
@@ -763,7 +764,8 @@ module Bln
         cr.move_to(@left.to_i, @top.to_i)
         cr.show_pango_layout(@layout)
       end
-      @window.set_shape(cr)
+      @window.set_shape(cr, @reshape)
+      @reshape = false
     end
 
     def get_state
