@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001-2004 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004-2017 by Shyouzou Sugitani <shy@users.osdn.me>
+#  Copyright (C) 2004-2019 by Shyouzou Sugitani <shy@users.osdn.me>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -442,7 +442,7 @@ module NGM
         search_forward()
       end
       item.add_accelerator('activate', accelgroup, Gdk::Keyval::KEY_F3,
-                           nil,
+                           Gdk::ModifierType.new(),
                            Gtk::AccelFlags::VISIBLE)
       menu.add(item)
       item = Gtk::SeparatorMenuItem.new()
