@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2003-2017 by Shyouzou Sugitani <shy@users.osdn.me>
+#  Copyright (C) 2003-2019 by Shyouzou Sugitani <shy@users.osdn.me>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -20,18 +20,15 @@ module Pix
 
   def self.surface_to_region(surface)
     region = Cairo::Region.new()
-    data = surface.data
     width = surface.width
-    pix_na = NArray.to_na(data, NArray::BYTE)
+    pix_na = NArray.to_na(surface.data, NArray::BYTE)
     pix_na.reshape!(4, pix_na.size / 4)
-    alpha = pix_na[3, true]
-    curr_y = -1
-    start_x = -1
-    end_x = -1
-    (alpha.ne 0).where.each {|i|
-      y = (i / width)
-      x = (i % width)
-      if start_x != -1
+    curr_y = nil
+    start_x = nil
+    end_x = nil
+    pix_na[3, true].where.each {|i|
+      y, x = i.divmod(width)
+      unless start_x.nil?
         if y != curr_y or x != (end_x + 1)
           region.union!(start_x, curr_y, end_x - start_x + 1, 1)
           curr_y = y
@@ -43,18 +40,55 @@ module Pix
       end
       end_x = x
     }
-    region.union!(start_x, curr_y, end_x - start_x + 1, 1) unless start_x == -1
+    region.union!(start_x, curr_y, end_x - start_x + 1, 1) unless start_x.nil?
+    return region
+  end
+
+  def self.surface_to_region_with_hints(surface, device_extents)
+    device_x1, device_y1, device_x2, device_y2 = device_extents
+    region = Cairo::Region.new()
+    width = surface.width
+    height = surface.height
+    stride = surface.stride
+    device_x1 =[0, [device_x1, width - 1].min].max
+    device_x2 =[0, [device_x2, width - 1].min].max
+    device_y1 =[0, [device_y1, height - 1].min].max
+    device_y2 =[0, [device_y2, height - 1].min].max
+    device_w = device_x2 - device_x1 + 1
+    device_h = device_y2 - device_y1 + 1
+    pix_na = NArray.to_na(surface.data[device_y1 * stride, device_h * stride], NArray::BYTE)
+    pix_na.reshape!(4, width, device_h)
+    pix_na = pix_na.slice(3, device_x1..device_x2, true)
+    curr_y = nil
+    start_x = nil
+    end_x = nil
+    pix_na.where.each {|i|
+      y, x = i.divmod(device_w)
+      unless start_x.nil?
+        if y != curr_y or x != (end_x + 1)
+          region.union!(start_x, curr_y, end_x - start_x + 1, 1)
+          curr_y = y
+          start_x = x
+        end
+      else
+        curr_y = y
+        start_x = x
+      end
+      end_x = x
+    }
+    region.union!(start_x, curr_y, end_x - start_x + 1, 1) unless start_x.nil?
+    region.translate!(device_x1, device_y1)
     return region
   end
 
   class BaseTransparentWindow < Gtk::Window
     alias :base_move :move
-    attr_reader :workarea
+    attr_reader :supports_alpha
 
     def initialize(type: Gtk::WindowType::TOPLEVEL)
       super(type)
       set_decorated(false)
-      set_resizable(false)
+      #set_resizable(false)
       signal_connect("screen-changed") do |widget, old_screen|
         screen_changed(widget, :old_screen => old_screen)
         next true
@@ -71,8 +105,7 @@ module Pix
       end
       @supports_alpha = composited?
       fail "assert" unless not visual.nil?
-      workarea = screen.display.primary_monitor.workarea # XXX
-      @workarea = [workarea.x, workarea.y, workarea.width, workarea.height]
+      maximize # not fullscreen
     end
   end
 
@@ -85,50 +118,30 @@ module Pix
       super()
       set_app_paintable(true)
       set_focus_on_map(false)
-      @__position = [0, 0]
       @__surface_position = [0, 0]
-      signal_connect_after('size_allocate') do |a|
-        size_allocate(a)
-        next false
-      end
+      @prev_position = [0, 0]
       # create drawing area
       @darea = Gtk::DrawingArea.new
+      @darea.set_size_request(*size) # XXX
       @darea.show()
       add(@darea)
-    end
-
-    def update_size(w, h)
-      @darea.set_size_request(w, h) # XXX
-      queue_resize()
-    end
-
-    def size_allocate(allocation)
-      new_x, new_y = @__position
-      base_move(new_x, new_y)
+      @region = nil
+      @device_extents = nil
     end
 
     def move(x, y)
-      left, top, scrn_w, scrn_h = @workarea
-      w, h = @darea.size_request() # XXX
-      new_x = [[left, x].max, left + scrn_w - w].min
-      new_y = [[top, y].max, top + scrn_h - h].min
-      base_move(new_x, new_y)
-      @__position = [new_x, new_y]
       @__surface_position = [x, y]
       @darea.queue_draw()
     end
 
     def get_draw_offset
-      window_x, window_y = @__position
-      surface_x, surface_y = @__surface_position
-      return surface_x - window_x, surface_y - window_y
+      return @__surface_position
     end
 
     def winpos_to_surfacepos(x, y, scale)
-      window_x, window_y = @__position
       surface_x, surface_y = @__surface_position
-      new_x = ((x - (surface_x - window_x)) * 100 / scale).to_i
-      new_y = ((y - (surface_y - window_y)) * 100 / scale).to_i
+      new_x = ((x - surface_x) * 100 / scale).to_i
+      new_y = ((y - surface_y) * 100 / scale).to_i
       return new_x, new_y
     end
 
@@ -145,24 +158,72 @@ module Pix
       cr.set_operator(Cairo::OPERATOR_SOURCE)
       # copy rectangle on the destination
       cr.rectangle(0, 0, surface.width, surface.height)
+      extents = cr.path_extents # cr.fill_extents
+      device_x1, device_y1 = cr.user_to_device(
+                   extents[0],
+                   extents[1])
+      device_x2, device_y2 = cr.user_to_device(
+                   extents[2],
+                   extents[3])
+      @device_extents = [device_x1, device_y1, device_x2, device_y2].map {|f| f.to_i}
       cr.fill()
       cr.restore()
     end
 
-    def set_shape(cr)
+    def set_shape(cr, reshape)
       return if RbConfig::CONFIG['host_os'] =~ /mswin|mingw|cygwin/
-      region = Pix.surface_to_region(cr.target.map_to_image)
+      if @region.nil? or reshape
+        if @device_extents.nil?
+          @region = Pix.surface_to_region(cr.target.map_to_image)
+        else
+          @region = Pix.surface_to_region_with_hints(cr.target.map_to_image, @device_extents)
+        end
+      else
+        dx, dy = @__surface_position.zip(@prev_position).map {|new, prev| new - prev}
+        @region.translate!(dx, dy)
+      end
+      @prev_position = @__surface_position
       if @supports_alpha
         input_shape_combine_region(nil)
-        input_shape_combine_region(region)
+        input_shape_combine_region(@region)
       else
         shape_combine_region(nil)
-        shape_combine_region(region)
+        shape_combine_region(@region)
       end
     end
   end
 
 
+  class TransparentApplicationWindow < Gtk::ApplicationWindow
+
+    def initialize(application)
+      super(application)
+      set_decorated(false)
+      set_app_paintable(true)
+      set_focus_on_map(false)
+      if composited?
+        input_shape_combine_region(Cairo::Region.new) # empty region
+      end
+      signal_connect("screen-changed") do |widget, old_screen|
+        screen_changed(widget, :old_screen => old_screen)
+        next true
+      end
+      screen_changed(self)
+    end
+
+    def screen_changed(widget, old_screen: nil)
+      if composited?
+        set_visual(screen.rgba_visual)
+      else
+        set_visual(screen.system_visual)
+        Logging::Logging.debug("screen does NOT support alpha.\n")
+      end
+      fail "assert" unless not visual.nil?
+      maximize # not fullscreen
+    end
+  end
+
+
   def self.get_png_size(path)
     return 0, 0 if not File.exists?(path)
     buf =
