@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2017 by Shyouzou Sugitani <shy@users.osdn.me>
+#  Copyright (C) 2002-2019 by Shyouzou Sugitani <shy@users.osdn.me>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -595,7 +595,7 @@ module Surface
     end
 
     def reset_position
-      left, top, scrn_w, scrn_h = get_workarea(0) # XXX
+      left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
       s0x, s0y, s0w, s0h = 0, 0, 0, 0 # XXX
       for side in 0..@window.length-1
         align = get_alignment(side)
@@ -628,14 +628,6 @@ module Surface
       end
     end
 
-    def get_workarea(side)
-      if @window.length > side
-        return @window[side].get_window.workarea
-      else
-        return [0, 0, 0, 0]
-      end
-    end
-
     def set_position(side, x, y)
       if @window.length > side
         @window[side].set_position(x, y)
@@ -844,7 +836,7 @@ module Surface
         @mikire = @kasanari = 0
         return
       end
-      left, top, scrn_w, scrn_h = get_workarea(0)
+      left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
       x0, y0 = get_position(0)
       s0w, s0h = get_surface_size(0)
       if (x0 + s0w / 3) < left or (x0 + s0w * 2 / 3) > (left + scrn_w) or \
@@ -911,6 +903,7 @@ module Surface
       @y_root = nil
       @click_count = 0
       @__balloon_offset = nil
+      @reshape = true
       @window.signal_connect('leave_notify_event') do |w, e|
         next window_leave_notify(w, e) # XXX
       end
@@ -973,8 +966,9 @@ module Surface
 
     def display_changed(screen)
       return unless @side.zero?
+      @reshape = true # XXX
       @parent.handle_request('NOTIFY', 'reset_position') # XXX
-      left, top, scrn_w, scrn_h = @window.workarea
+      left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
       @parent.handle_request(
         'NOTIFY', 'notify_event', 'OnDisplayChange',
         Gdk.Visual.get_best_depth(), scrn_w, scrn_h)
@@ -1097,6 +1091,7 @@ module Surface
       else
         @surface_id = surface_id
       end
+      @reshape = true
       @seriko.reset(self, surface_id)
       # define collision areas
       @collisions = @region[@surface_id]
@@ -1108,7 +1103,7 @@ module Surface
       case get_alignment()
       when 0
         yoffset = (dh - h)
-        left, top, scrn_w, scrn_h = @window.workarea
+        left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
         y = (top + scrn_h - dh)
       when 1
         yoffset = 0
@@ -1116,8 +1111,6 @@ module Surface
         yoffset = ((dh - h) / 2)
       end
       @window_offset = [xoffset, yoffset]
-      # resize window
-      @window.update_size(*get_max_size())
       @seriko.start(self)
       # relocate window
       unless @dragged # XXX
@@ -1253,7 +1246,7 @@ module Surface
           font_desc.set_size(8 * Pango::SCALE)
           layout = cr.create_pango_layout
           layout.set_font_description(font_desc)
-          layout.set_wrap(Pango::WrapMode::WORD_CHAR) # XXX
+          layout.set_wrap(:char) # XXX
           layout.set_text(part)
           cr.show_pango_layout(layout)
         end
@@ -1306,6 +1299,7 @@ module Surface
 
     def update_frame_buffer
       return if @parent.handle_request('GET', 'lock_repaint')
+      @reshape = true # FIXME: depends on Seriko
       new_surface = create_image_surface(@seriko.get_base_id)
       fail "assert" if new_surface.nil?
       # update collision areas
@@ -1347,7 +1341,8 @@ module Surface
       unless @parent.handle_request('GET', 'get_preference', 'check_collision').zero?
         draw_region(cr)
       end
-      @window.set_shape(cr)
+      @window.set_shape(cr, @reshape)
+      @reshape = false
     end
 
     def remove_overlay(actor)
@@ -1388,7 +1383,7 @@ module Surface
     end
 
     def get_max_size
-      left, top, scrn_w, scrn_h = @window.workarea
+      left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
       w, h = @maxsize
       scale = get_scale
       w = [scrn_w, [8, (w * scale / 100).to_i].max].min
@@ -1466,7 +1461,7 @@ module Surface
       @position = [x, y]
       new_x, new_y = get_position()
       @window.move(new_x, new_y)
-      left, top, scrn_w, scrn_h = @window.workarea
+      left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
       if x > (left + scrn_w / 2)
         new_direction = 0
       else
@@ -1503,13 +1498,13 @@ module Surface
       return if @dragged # XXX: position will be reset after button release event
       case align
       when 0
-        left, top, scrn_w, scrn_h = @window.workarea
+        left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
         sw, sh = get_max_size()
         sx, sy = @position # XXX: without window_offset
         sy = (top + scrn_h - sh)
         set_position(sx, sy)
       when 1
-        left, top, scrn_w, scrn_h = @window.workarea
+        left, top, scrn_w, scrn_h = @parent.handle_request('GET', 'get_workarea')
         sx, sy = @position # XXX: without window_offset
         sy = top
         set_position(sx, sy)
@@ -1534,6 +1529,7 @@ module Surface
     def show
       return if @parent.handle_request('GET', 'lock_repaint')
       return if @__shown
+      @reshape = true
       @__shown = true
       x, y = get_position()
       @window.move(x, y) # XXX: call before showing the window
@@ -1667,9 +1663,7 @@ module Surface
                                  @side, x, y, part)
         end
       end
-      if event.is_hint == 1
-        Gdk::Event.request_motions(event)
-      end
+      Gdk::Event.request_motions(event) if event.is_hint == 1
       return true
     end
 
