@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2018 by Shyouzou Sugitani <shy@users.osdn.me>
+#  Copyright (C) 2002-2019 by Shyouzou Sugitani <shy@users.osdn.me>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -82,58 +82,6 @@ module Ninix_Main
     fail SystemExit
   end
   
-  def self.main(option)
-    # parse command line arguments
-    Logging::Logging.add_logger(Logger.new(option[:logfile])) unless option[:logfile].nil?
-    # TCP 7743：伺か（未使用）(IANA Registered Port for SSTP)
-    # UDP 7743：伺か（未使用）(IANA Registered Port for SSTP)
-    # TCP 9801：伺か          (IANA Registered Port for SSTP)
-    # UDP 9801：伺か（未使用）(IANA Registered Port for SSTP)
-    # TCP 9821：SSP
-    # TCP 11000：伺か（廃止） (IANA Registered Port for IRISA)
-    sstp_port = [9801]
-    # parse command line arguments
-    unless option[:sstp_port].nil?
-      if option[:sstp_port].to_i < 1024
-        Logging::Logging.warning("Invalid --sstp-port number (ignored)")
-      else
-        sstp_port << option[:sstp_port].to_i
-      end
-    end
-    Logging::Logging.set_level(Logger::DEBUG) unless option[:debug].nil?
-    home_dir = Home.get_ninix_home()
-    unless File.exists?(home_dir)
-      begin
-        FileUtils.mkdir_p(home_dir)
-      rescue
-        raise SystemExit("Cannot create Home directory (abort)\n")
-      end
-    end
-    lockfile_path = File.join(Home.get_ninix_home(), ".lock")
-    if File.exists?(lockfile_path)
-      f = open(lockfile_path, 'r')
-      abend = f.gets
-    else
-      abend = nil
-    end
-    # aquire Inter Process Mutex (not Global Mutex)
-    f = open(lockfile_path, 'w')
-    begin
-      Lock.lockfile(f)
-    rescue
-      raise SystemExit("ninix-aya is already running")
-    end
-    # start
-    app = Application.new(f, :sstp_port => sstp_port)
-    app.run(abend)
-    f.truncate(0)
-    begin
-      Lock.unlockfile(f)
-    rescue
-      #pass
-    end
-  end
-  
   class SSTPControler < MetaMagic::Holon
 
     def initialize(sstp_port)
@@ -416,6 +364,7 @@ module Ninix_Main
         'reset_sstp_flag' => 'reset_sstp_flag',
         'get_sstp_port' => 'get_sstp_port',
         'get_prefix' => 'get_sakura_prefix',
+        'get_workarea' => 'get_workarea'
       }
       unless handlers.include?(event)
         if Application.method_defined?(event)
@@ -877,8 +826,9 @@ module Ninix_Main
       return nil
     end
 
-    def run(abend)
+    def run(abend, app_window)
       @abend = abend
+      @app_window = app_window
       if RbConfig::CONFIG['host_os'] =~ /mswin|mingw|cygwin/
         # The SIGTERM signal is not generated under Windows NT.
         # Win32::Console::API.SetConsoleCtrlHandler will probably not be implemented.
@@ -1310,6 +1260,10 @@ module Ninix_Main
       end
       return true
     end
+
+    def get_workarea
+      [0, 0, *@app_window.size]
+    end
   end
 
   class Console
@@ -1690,15 +1644,73 @@ end
 
 Logging::Logging.set_level(Logger::INFO)
 
-opt = OptionParser.new
-option = {}
-opt.on('--sstp-port sstp_port', 'additional port for listening SSTP requests') {|v| option[:sstp_port] = v}
-opt.on('--debug', 'debug') {|v| option[:debug] = v}
-opt.on('--logfile logfile_name', 'logfile name') {|v| option[:logfile] = v}
-opt.parse!(ARGV)
+gtk_app = Gtk::Application.new('net.osdn.ninix-aya', :flags_none)
+
+gtk_app.signal_connect 'activate' do |application|
+  # parse command line arguments
+  opt = OptionParser.new
+  option = {}
+  opt.on('--sstp-port sstp_port', 'additional port for listening SSTP requests') {|v| option[:sstp_port] = v}
+  opt.on('--debug', 'debug') {|v| option[:debug] = v}
+  opt.on('--logfile logfile_name', 'logfile name') {|v| option[:logfile] = v}
+  opt.parse!(ARGV)
+  Logging::Logging.add_logger(Logger.new(option[:logfile])) unless option[:logfile].nil?
+  # TCP 7743：伺か（未使用）(IANA Registered Port for SSTP)
+  # UDP 7743：伺か（未使用）(IANA Registered Port for SSTP)
+  # TCP 9801：伺か          (IANA Registered Port for SSTP)
+  # UDP 9801：伺か（未使用）(IANA Registered Port for SSTP)
+  # TCP 9821：SSP
+  # TCP 11000：伺か（廃止） (IANA Registered Port for IRISA)
+  sstp_port = [9801]
+  # parse command line arguments
+  unless option[:sstp_port].nil?
+    if option[:sstp_port].to_i < 1024
+      Logging::Logging.warning("Invalid --sstp-port number (ignored)")
+    else
+      sstp_port << option[:sstp_port].to_i
+    end
+  end
+  Logging::Logging.set_level(Logger::DEBUG) unless option[:debug].nil?
+  home_dir = Home.get_ninix_home()
+  unless File.exists?(home_dir)
+    begin
+      FileUtils.mkdir_p(home_dir)
+    rescue
+      raise SystemExit("Cannot create Home directory (abort)\n")
+    end
+  end
+  lockfile_path = File.join(Home.get_ninix_home(), ".lock")
+  if File.exists?(lockfile_path)
+    f = open(lockfile_path, 'r')
+    abend = f.gets
+  else
+    abend = nil
+  end
+  # aquire Inter Process Mutex (not Global Mutex)
+  f = open(lockfile_path, 'w')
+  begin
+    Lock.lockfile(f)
+  rescue
+    raise SystemExit("ninix-aya is already running")
+  end
+  app_window = Pix::TransparentApplicationWindow.new(application)
+  app_window.set_title("Ninix-aya")
+  app_window.show_all
+  # start
+  app = Ninix_Main::Application.new(f, :sstp_port => sstp_port)
+  app.run(abend, app_window)
+  # end
+  f.truncate(0)
+  begin
+    Lock.unlockfile(f)
+  rescue
+    #pass
+  end
+  app_window.destroy
+end
 
 begin
-  Ninix_Main.main(option)
+  gtk_app.run
 rescue => e # should never rescue Exception
   Ninix_Main.handleException(e)
 end
